@@ -11,6 +11,7 @@ import ssl
 
 import pytest
 import hypothesis
+import hypothesis.database
 
 pytest.register_assert_rewrite('helpers')
 
@@ -33,10 +34,19 @@ _qute_scheme_handler = None
 
 
 # Set hypothesis settings
+hypotheses_optional_kwargs = {}
+if "HYPOTHESIS_EXAMPLES_DIR" in os.environ:
+    hypotheses_optional_kwargs[
+        "database"
+    ] = hypothesis.database.DirectoryBasedExampleDatabase(
+        os.environ["HYPOTHESIS_EXAMPLES_DIR"]
+    )
+
 hypothesis.settings.register_profile(
     'default', hypothesis.settings(
         deadline=600,
         suppress_health_check=[hypothesis.HealthCheck.function_scoped_fixture],
+        **hypotheses_optional_kwargs,
     )
 )
 hypothesis.settings.register_profile(
@@ -45,7 +55,8 @@ hypothesis.settings.register_profile(
         suppress_health_check=[
             hypothesis.HealthCheck.function_scoped_fixture,
             hypothesis.HealthCheck.too_slow
-        ]
+        ],
+        **hypotheses_optional_kwargs,
     )
 )
 hypothesis.settings.load_profile('ci' if testutils.ON_CI else 'default')
@@ -361,7 +372,8 @@ def pytest_runtest_makereport(item, call):
 
 @pytest.hookimpl(hookwrapper=True)
 def pytest_terminal_summary(terminalreporter):
-    """Group benchmark results on CI."""
+    """Add custom pytest summary sections."""
+    # Group benchmark results on CI.
     if testutils.ON_CI:
         terminalreporter.write_line(
             testutils.gha_group_begin('Benchmark results'))
@@ -369,3 +381,21 @@ def pytest_terminal_summary(terminalreporter):
         terminalreporter.write_line(testutils.gha_group_end())
     else:
         yield
+
+    # List any screenshots of failed end2end tests that were generated during
+    # the run. Screenshots are captured from QuteProc.after_test()
+    properties = lambda report: dict(report.user_properties)
+    reports = [
+        report
+        for report in terminalreporter.getreports("")
+        if "screenshot" in properties(report)
+    ]
+    screenshots = [
+        pathlib.Path(properties(report)["screenshot"])
+        for report in reports
+    ]
+
+    if screenshots:
+        terminalreporter.ensure_newline()
+        screenshot_dir = screenshots[0].parent
+        terminalreporter.section(f"End2end screenshots available in: {screenshot_dir}", sep="-", blue=True, bold=True)
